/*
 * Copyright (c) 1993-2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** platineGetPosTaskCodels.c
 **
 ** Codels appeles par la tache platineGetPosTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#include <math.h>
#include <stdio.h>

#include <portLib.h>
#include <taskLib.h>
#include "h2mathLib.h"

#include "server/platineHeader.h"
#include "driver/ptClient.h"
#include "platineCodels.h"

BOOL platineInitPtParams(void)
{
  double val=0;
  int ival=0;
  PT_CMD_STATUS status;
  PT_ERR err;
  char *funcName = "platineInitPtParams";
  PLATINE_AXIS axis;

  printf ("platineInitPtParams\n");
  if(ptDriverSDI == NULL ||
     !(ptDriverSDI->ptDriverInitialized)) {
    printf ("ptDriver not initialized\n");
    return FALSE;
  }
  
  SDI_F->constConvert.ptResolution = ptDriverSDI->ptResolution;
  SDI_F->constConvert.step2rad = ptDriverSDI->ptStep2rad;
  SDI_F->constConvert.rad2step = ptDriverSDI->ptRad2step;
  SDI_F->constConvert.step2deg = ptDriverSDI->ptStep2deg;
  SDI_F->constConvert.deg2step = ptDriverSDI->ptDeg2step;

/*   if((status = ptSendAndGet (PT_ECHO_OFF, &ival, TRUE, &err, ptDriverSDI)) != PT_REPLY_OK) { */
/*     ptPrint(stdout, funcName, PT_ECHO_OFF, status, err, ptDriverSDI); */
/*     return FALSE; */
/*   } */
    
  /**
   ** PARAMETRES CONSTANTS 
   **/ 

  /* PAN */
  axis = PLATINE_PAN;

  if ((status = ptGetInt(GET_MIN_POS_PAN, &ival, &err, ptDriverSDI)) 
      != PT_CONST_OK) {
    ptPrint(stdout, funcName, GET_MIN_POS_PAN, status, err, 
	    ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.pan.posMin = PT_STEP_TO_RAD(ival, 
							  ptDriverSDI);
  SDI_F->stepAxisParams[axis].posMin = ival;
  
  if ((status = ptGetInt(GET_MAX_POS_PAN, &ival, &err, ptDriverSDI)) 
      != PT_CONST_OK) {
    ptPrint(stdout, funcName, GET_MAX_POS_PAN, status, err, 
	    ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.pan.posMax = PT_STEP_TO_RAD(ival, 
							  ptDriverSDI);
  SDI_F->stepAxisParams[axis].posMax = ival;
  
  /* TILT */
  axis = PLATINE_TILT;

  if ((status = ptGetInt(GET_MIN_POS_TILT, &ival, &err, ptDriverSDI)) 
      != PT_CONST_OK) {
    ptPrint(stdout, funcName, GET_MIN_POS_TILT, status, err, 
	    ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.tilt.posMin = PT_STEP_TO_RAD(ival, 
							   ptDriverSDI);
  SDI_F->stepAxisParams[axis].posMin = ival;

  if ((status = ptGetInt(GET_MAX_POS_TILT, &ival, &err, ptDriverSDI))
      != PT_CONST_OK) {
    ptPrint(stdout, funcName, GET_MAX_POS_TILT, status, err, 
	    ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.tilt.posMax = PT_STEP_TO_RAD(ival, 
							   ptDriverSDI);
  SDI_F->stepAxisParams[axis].posMax = ival;


  /**
   ** PARAMETRES MODIFIABLES 
   **/ 

  /* PAN */

  axis = PLATINE_PAN;

  /* Min Speed */
  status = ptSendAndGet(GET_MIN_SPEED_PAN, &ival, TRUE, &err, 
			ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_MIN_SPEED_PAN, status, err, 
	    ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.pan.speedMinMin = 
    PT_STEP_TO_RAD(ival, ptDriverSDI);
  SDI_F->platineParams.speedProfile.pan.minSpeed = 
    PT_STEP_TO_RAD(ival, ptDriverSDI);
  SDI_F->stepAxisParams[axis].speedMinMin = ival;
  SDI_F->stepAxisParams[axis].minSpeed = ival;

  /* Max speed */
  status = ptSendAndGet(GET_MAX_SPEED_PAN, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_MAX_SPEED_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.pan.speedMaxMax = PT_STEP_TO_RAD(ival,ptDriverSDI);
  SDI_F->platineParams.speedProfile.pan.maxSpeed = PT_STEP_TO_RAD(ival,ptDriverSDI);
  SDI_F->stepAxisParams[axis].speedMaxMax = ival;
  SDI_F->stepAxisParams[axis].maxSpeed = ival;
  
  /* Base speed */
  status = ptSendAndGetRad(GET_BASE_SPEED_PAN, &val, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_BASE_SPEED_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.speedProfile.pan.baseSpeed = val;
  
  /* Speed */
  status = ptSendAndGetRad(GET_SPEED_PAN, &val, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_SPEED_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.speedProfile.pan.speed = val;
  
  /* Accel */
  status = ptSendAndGetRad(GET_ACCEL_PAN, &val, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_ACCEL_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.speedProfile.pan.accel = val;
  
  /* TILT */
  
  axis = PLATINE_TILT;
  
  /* Min Speed */
  status = ptSendAndGet(GET_MIN_SPEED_TILT, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_MIN_SPEED_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.tilt.speedMinMin = PT_STEP_TO_RAD(ival,ptDriverSDI);
  SDI_F->platineParams.speedProfile.tilt.minSpeed = PT_STEP_TO_RAD(ival,ptDriverSDI);
  SDI_F->stepAxisParams[axis].speedMinMin = ival;
  SDI_F->stepAxisParams[axis].minSpeed = ival;

  /* Max speed */
  status = ptSendAndGet(GET_MAX_SPEED_TILT, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_MAX_SPEED_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.consts.tilt.speedMaxMax = PT_STEP_TO_RAD(ival,ptDriverSDI);
  SDI_F->platineParams.speedProfile.tilt.maxSpeed = PT_STEP_TO_RAD(ival,ptDriverSDI);
  SDI_F->stepAxisParams[axis].speedMaxMax = ival;
  SDI_F->stepAxisParams[axis].maxSpeed = ival;

  /* Base speed */
  status = ptSendAndGetRad(GET_BASE_SPEED_TILT, &val, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_BASE_SPEED_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.speedProfile.tilt.baseSpeed = val;
  
  /* Speed */
  status = ptSendAndGetRad(GET_SPEED_TILT, &val, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_SPEED_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.speedProfile.tilt.speed = val;

  /* Accel */
  status = ptSendAndGetRad(GET_ACCEL_TILT, &val, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_ACCEL_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  SDI_F->platineParams.speedProfile.tilt.accel = val;

  /**
   ** PARAMETRES PUISSANCE 
   **/ 

  /* Puissance a l'arret PAN */
  status = ptSendAndGet(GET_HOLD_POWER_PAN, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_HOLD_POWER_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
  switch (ival) {
  case PT_REGULAR_POWER:
    SDI_F->platineParams.power.pan.holdPowerMode = PLATINE_HOLD_POWER_REGULAR;
    break;
  case PT_LOW_POWER:
    SDI_F->platineParams.power.pan.holdPowerMode = PLATINE_HOLD_POWER_LOW;
    break;
  case PT_POWER_OFF:
    SDI_F->platineParams.power.pan.holdPowerMode = PLATINE_HOLD_POWER_OFF;
    break;
  default:
    ptPrint(stdout, funcName, GET_HOLD_POWER_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
 
  /* Puissance en mouvement PAN */
  status = ptSendAndGet(GET_MOVE_POWER_PAN, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_MOVE_POWER_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
  switch (ival) {
  case PT_REGULAR_POWER:
    SDI_F->platineParams.power.pan.holdPowerMode = PLATINE_MOVE_POWER_REGULAR;
    break;
  case PT_LOW_POWER:
    SDI_F->platineParams.power.pan.holdPowerMode = PLATINE_MOVE_POWER_LOW;
    break;
  case PT_HIGH_POWER:
    SDI_F->platineParams.power.pan.holdPowerMode = PLATINE_MOVE_POWER_HIGH;
    break;
  default:
    ptPrint(stdout, funcName, GET_MOVE_POWER_PAN, status, err, ptDriverSDI);
    return FALSE;
  }
 
  /* Puissance a l'arret TILT */
  status = ptSendAndGet(GET_HOLD_POWER_TILT, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_HOLD_POWER_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  switch (ival) {
  case PT_REGULAR_POWER:
    SDI_F->platineParams.power.tilt.holdPowerMode = PLATINE_HOLD_POWER_REGULAR;
    break;
  case PT_LOW_POWER:
    SDI_F->platineParams.power.tilt.holdPowerMode = PLATINE_HOLD_POWER_LOW;
    break;
  case PT_POWER_OFF:
    SDI_F->platineParams.power.tilt.holdPowerMode = PLATINE_HOLD_POWER_OFF;
    break;
  default:
    ptPrint(stdout, funcName, GET_HOLD_POWER_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
 
  /* Puissance en mouvement TILT */
  status = ptSendAndGet(GET_MOVE_POWER_TILT, &ival, TRUE, &err, ptDriverSDI);
  if (status != PT_REPLY_OK) {
    ptPrint(stdout, funcName, GET_MOVE_POWER_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  switch (ival) {
  case PT_REGULAR_POWER:
    SDI_F->platineParams.power.tilt.holdPowerMode = PLATINE_MOVE_POWER_REGULAR;
    break;
  case PT_LOW_POWER:
    SDI_F->platineParams.power.tilt.holdPowerMode = PLATINE_MOVE_POWER_LOW;
    break;
  case PT_HIGH_POWER:
    SDI_F->platineParams.power.tilt.holdPowerMode = PLATINE_MOVE_POWER_HIGH;
    break;
  default:
    ptPrint(stdout, funcName, GET_MOVE_POWER_TILT, status, err, ptDriverSDI);
    return FALSE;
  }
  return TRUE;
}

int platineConvertPtErr(PT_ERR err)
{
  switch (err) {
  case PT_OUT_OF_RANGE:
    return S_platine_PT_OUT_OF_RANGE;
  case PT_SMALL_PAN_VALUE:
    return S_platine_PT_SMALL_PAN_VALUE;
  case PT_SMALL_TILT_VALUE:
    return S_platine_PT_SMALL_TILT_VALUE;
  case PT_ILLEGAL_COMMAND:
    return S_platine_PT_ILLEGAL_COMMAND;
  case PT_ILLEGAL_ACCEL:
    return S_platine_PT_ILLEGAL_ACCEL;
  case PT_SYS_ERR:
    return S_platine_PT_SYS_ERR;
  case PT_STOP_ENDS:
    return S_platine_PT_STOP_ENDS;
  case PT_NO_ERR:
    return S_platine_PT_NO_ERR;
  case PT_TIMEOUT_ERR:
    return S_platine_PT_TIMEOUT;
  case PT_UNKNOWN_ERR:
  default:
    return S_platine_PT_UNKNOWN_ERR;
  }
}

