/**
 ** platineGetPosTaskCodels.c
 **
 ** Codels appeles par la tache platineGetPosTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#ifndef UNIX
#include <vxWorks.h>
#include <taskLib.h>
#else
#include <portLib.h>
#endif
#include <stdio.h>

#include "h2mathLib.h"

#include "platineMsgLib.h"
#include "platinePosterLib.h"

BOOL testTrackPtInit = FALSE;
static  POSTER_ID trackPosterId;
static  CLIENT_ID trackClientId;

typedef enum TEST_PT_MODE {
  MSG,
  POSTER_POS,
  POSTER_SPEED
} TEST_PT_MODE;

void testTrackPt(int sleepTic, double panDeg, double tiltDeg, 
		 double speedDeg, int mode)
{
  PLATINE_STATE state;
  PLATINE_AXIS_STR ref, refSpeed;
  double dt, da;
  double pan;
  double tilt;
  char *trackPosterName = "testTrackPtPoster";
  char *mboxName = "testPtMbox";
  int rqstId, activity, bilan;
  PLATINE_REF_COORD_POS_STR refCoord;

  /**
   ** INIT 
   **/

  /* Init posters, mbox, ... */
  if (!testTrackPtInit) {
    if (posterCreate(trackPosterName, sizeof(PLATINE_AXIS_STR), 
		     &trackPosterId) == ERROR) {
      printf ("Cannot create poster \n");
      return;
    }
    if (platinePosterInit() == ERROR) {
      printf ("Cannot init poster platine\n");
      return;
    }

    if (csMboxInit (mboxName, 0, PLATINE_CLIENT_MBOX_REPLY_SIZE) != OK) {
      h2perror("");
      printf ("Init mbox %s failed\n", mboxName);
      return;
    }
    if (platineClientInit(&trackClientId))  {
      printf ("Cannot init client platine\n");
      return;
    }
    printf ("Init ok\n");
    testTrackPtInit =  TRUE;
  }
  
  /* Init ref */
  if (platineStatestateRadPosterRead(&state) == ERROR) {
    printf ("Cannot read poster platine\n");
    return;
  }

  ref.pan = state.pos.pan;
  ref.tilt = state.pos.tilt;
  refCoord.pan = ref.pan;
  refCoord.tilt = ref.tilt;
  refCoord.unity = PLATINE_RAD;
  refCoord.survEndFlag = PLATINE_FALSE;
  refSpeed.pan = 0.0;
  refSpeed.tilt = 0.0;

  if (mode == POSTER_POS) {
    posterWrite(trackPosterId, 0, (void *) &ref, sizeof(PLATINE_STATE));
    if(platineTrackPosRqstAndAck(trackClientId, &rqstId, 0, trackPosterName, 
				 &activity, &bilan) != WAITING_FINAL_REPLY) {
      h2perror("platineTrackPosRqstAndAck");
      return;
    }
  }
  else if (mode == POSTER_SPEED) {
    posterWrite(trackPosterId, 0, (void *) &refSpeed, sizeof(PLATINE_STATE));
    if(platineTrackSpeedRqstAndAck(trackClientId, &rqstId, 0, 
				   trackPosterName, &activity, &bilan) 
       != WAITING_FINAL_REPLY) {
      h2perror("platineTrackPosRqstAndAck");
      return;
    }
  }    
   
  /* Compute traj */
  pan = 0.0;
  tilt = 0.0;
  dt = sleepTic * 0.005;
  da = speedDeg * dt;
  printf ("pan %f deg   tilt %f deg   --   da %f deg   dt %f sec\n", 
	  RAD_TO_DEG(ref.pan), RAD_TO_DEG(ref.tilt), da, dt);

  refSpeed.pan = DEG_TO_RAD(speedDeg);
  refSpeed.tilt = DEG_TO_RAD(speedDeg);

  /* LOOP */
  while (fabs(pan) < fabs(panDeg) || fabs(tilt) < fabs(tiltDeg)) {

    if (fabs(pan) < fabs(panDeg)) {
      pan += da;
      ref.pan += DEG_TO_RAD(da);
      refCoord.pan = ref.pan;
    }

    if (fabs(tilt) < fabs(tiltDeg)) {
      tilt += da;
      ref.tilt += DEG_TO_RAD(da);
      refCoord.tilt = ref.tilt;
    }

    if (mode == POSTER_POS) 
      posterWrite(trackPosterId, 0, (void *) &ref, sizeof(PLATINE_STATE));
    else if (mode == POSTER_SPEED)
      posterWrite(trackPosterId, 0, (void *) &refSpeed, sizeof(PLATINE_STATE));
    else
      platineCmdPosCoordRqstAndRcv (trackClientId, 0, &refCoord, 
				    &activity, &bilan);

    taskDelay(sleepTic);

  } /* LOOP */

  /* Arret tracking */
  if (mode == POSTER_POS) {
    if(platineAbortRqstAndRcv(trackClientId, &activity, &bilan) 
       != FINAL_REPLY_OK || bilan != OK) {
      h2perror("platineAbortRqstAndRcv");
    }

    if(platineTrackPosReplyRcv(trackClientId, rqstId, BLOCK_ON_FINAL_REPLY,
			       &activity, &bilan)  != FINAL_REPLY_OK || bilan != OK) {
      h2perror("platineTrackPosReplyRcv");
    }
  }
  else if (mode == POSTER_SPEED) {
    refSpeed.pan = 0.0;
    refSpeed.tilt = 0.0;
    posterWrite(trackPosterId, 0, (void *) &refSpeed, sizeof(PLATINE_STATE));
    taskDelay(20);
    if(platineAbortRqstAndRcv(trackClientId, &activity, &bilan) 
       != FINAL_REPLY_OK || bilan != OK) {
      h2perror("platineAbortRqstAndRcv");
    }
    
    if(platineTrackSpeedReplyRcv(trackClientId, rqstId, BLOCK_ON_FINAL_REPLY,
				 &activity, &bilan)  != FINAL_REPLY_OK || bilan != OK) {
      h2perror("platineTrackPosReplyRcv");
    }
  }
  

  taskDelay(50);
  posterDelete(trackPosterId);
  platineClientEnd (trackClientId);
  csMboxEnd ();
  testTrackPtInit = FALSE;

  printf ("end\n");
}
