#ifndef PT_DRIVER_H
#define PT_DRIVER_H

/*----------------------------------------------------------------------
 * Definition of the driver internal data structure
 */

#define PT_DRIVER_NAME_LENGTH 32

typedef struct PT_DRIVER_STR {
  char ptDriverName[PT_DRIVER_NAME_LENGTH];
  int ptVerbose;
  int v0, v1, v2;      /* version v0.v1.v2 */
  double ptResolution; /* Resolution des codeurs (sec arc / position)
			  Utilisee dans les macros PT_STEP_TO_RAD ... */
  double ptStep2rad;
  double ptRad2step;
  double ptStep2deg;
  double ptDeg2step;

  BOOL ptDriverInitialized;
  BOOL ptStopDriverFlag;

  struct PT_DRIVER_PRIVATE *privateData;
} PT_DRIVER_STR;


/*----------------------------------------------------------------------
 *    CONSTANTES
 */

/* Taille max des messages envoyes ou receptionnes par le driver */
#define PT_MAX_MSG_LENGTH 128

#define PT_ALL (-1)

/*----------------------------------------------------------------------*/
/* Timeout de reception des donnees */

#define PT_TIMEOUT (1000)  /* 2 lectures (SendAndGet) ou 4 lectures (autre) */
#define PT_NB_TICS_PER_CYCLE_IN_BLOCK_MODE 2

/*----------------------------------------------------------------------
 *     MACROS DE CONVERSION   steps <-> deg <-> rad
 */

#define PT_DEG_TO_STEP(deg,ptDriverSDI) ((int)floor(deg*ptDriverSDI->ptDeg2step+0.5))
#define PT_STEP_TO_DEG(step,ptDriverSDI) ((double)step*ptDriverSDI->ptStep2deg)

#define PT_RAD_TO_STEP(rad,ptDriverSDI) ((int)floor(rad*ptDriverSDI->ptRad2step+0.5))
#define PT_STEP_TO_RAD(step,ptDriverSDI) ((double)step*ptDriverSDI->ptStep2rad)


/*----------------------------------------------------------------------
 *    ETAT DES COMMANDES INTERPRETEES
 */

typedef enum PT_CMD_STATUS {
  PT_NO_CMD,         /* Pas de tel commande demandee */
  PT_WAITING_REPLY,   /* Commande acceptee.
			     En fait pour arriver la` on pourrait enabler
			     l'echo et le tester (commande "ee") */
  PT_REPLY_OK,       /* Valeur de retour receptionnee */
  PT_REPLY_ERROR,    /* Erreur dans la commande */
  PT_CMD_UNKNOWN,    /* Commande inconnue */
  PT_CONST_OK,       /* Constante deja recuperee (inutile de recommencer) */
  PT_REPLY_TIMEOUT
}PT_CMD_STATUS;

#define PT_CMD_STATUS_STRINGS {\
  "PT_NO_CMD", \
  "PT_WAITING_REPLY", \
  "PT_REPLY_OK", \
  "PT_REPLY_ERROR", \
  "PT_CMD_UNKNOWN", \
  "PT_CONST_OK", \
  "PT_REPLY_TIMEOUT" \
}

extern const char *ptCmdStatusStrings[];   /* Tableau initialise avec
					      PT_CMD_STATUS_STRINGS */

/*----------------------------------------------------------------------
 *    LISTE DES COMMANDES INTERPRETEES
 *
 *    (Si modif, maintenir la coherence avec les description des
 *     commandes ptCmdDesc dans ptDriverPrivate.h)
 */


typedef enum PT_CMD {
  /* PAN GET */
  GET_POS_ABS_PAN = 0,    /* 0 */
  GET_POS_REL_PAN,        /* 1 */
  GET_SPEED_PAN,          /* 2 */
  GET_ACCEL_PAN,          /* 3 */

  GET_BASE_SPEED_PAN,     /* 4 */
  GET_MIN_SPEED_PAN,      /* 5 */
  GET_MAX_SPEED_PAN,      /* 6 */

  GET_MIN_POS_PAN,        /* 7 */
  GET_MAX_POS_PAN,        /* 8 */
  GET_RES_PAN,            /* 9 */

  /* PAN SET */
  SET_POS_ABS_PAN,         /* 10 */
  SET_POS_REL_PAN,         /* 11 */
  SET_SPEED_PAN,           /* 12 */
  SET_ACCEL_PAN,           /* 13 */

  SET_BASE_SPEED_PAN,      /* 14 */
  SET_MIN_SPEED_PAN,       /* 15 */
  SET_MAX_SPEED_PAN,       /* 16 */

  /* TILT GET */
  GET_POS_ABS_TILT,        /* 17 */
  GET_POS_REL_TILT,        /* 18 */
  GET_SPEED_TILT,          /* 19 */
  GET_ACCEL_TILT,          /* 20 */

  GET_BASE_SPEED_TILT,     /* 21 */
  GET_MIN_SPEED_TILT,      /* 22 */
  GET_MAX_SPEED_TILT,      /* 23 */

  GET_MIN_POS_TILT,        /* 24 */
  GET_MAX_POS_TILT,        /* 25 */
  GET_RES_TILT,            /* 26 */

  /* TILT SET */
  SET_POS_ABS_TILT,        /* 27 */
  SET_POS_REL_TILT,        /* 28 */
  SET_SPEED_TILT,          /* 29 */
  SET_ACCEL_TILT,          /* 30 */

  SET_BASE_SPEED_TILT,     /* 31 */
  SET_MIN_SPEED_TILT,      /* 32 */
  SET_MAX_SPEED_TILT,      /* 33 */

  /* RESET, ECHO, LIMIT */
  PT_RESET,                /* 34 */

  PT_SYNCHO_MODE,          /* 35 */
  PT_IMMEDIAT_MODE,        /* 36 */
  PT_GO_SYNCH_OR_WAIT_IMM,  /* 37 */

  PT_GET_ECHO,             /* 38 */
  PT_ECHO_ON,              /* 39 */
  PT_ECHO_OFF,             /* 40 */

  PT_GET_LIMIT,            /* 41 */
  PT_LIMIT_ON,             /* 42 */
  PT_LIMIT_OFF,            /* 43 */

  PT_VERSION,              /* 44 */

  /* POWER MODE */
  GET_HOLD_POWER_PAN,          /* 45 */
  SET_HOLD_POWER_REGULAR_PAN,  /* 46 */
  SET_HOLD_POWER_LOW_PAN,      /* 47 */
  SET_HOLD_POWER_OFF_PAN,      /* 48 */

  GET_MOVE_POWER_PAN,          /* 49 */
  SET_MOVE_POWER_REGULAR_PAN,  /* 50 */
  SET_MOVE_POWER_LOW_PAN,      /* 51 */
  SET_MOVE_POWER_HIGH_PAN,     /* 52 */

  GET_HOLD_POWER_TILT,         /* 53 */
  SET_HOLD_POWER_REGULAR_TILT, /* 54 */
  SET_HOLD_POWER_LOW_TILT,     /* 55 */
  SET_HOLD_POWER_OFF_TILT,     /* 56 */

  GET_MOVE_POWER_TILT,         /* 57 */
  SET_MOVE_POWER_REGULAR_TILT, /* 58 */
  SET_MOVE_POWER_LOW_TILT,     /* 59 */
  SET_MOVE_POWER_HIGH_TILT,    /* 60 */

  /* SETTINGS */
  SAVE_SETTING,                /* 61 */
  RESTORE_SAVED_SETTING,       /* 62 */
  RESTORE_FACTORY_SETTING,     /* 63 */

  PT_ASCII,                    /* 64 */


  PT_HELP,                     /* 65 */

  /* NOUVELLE COMMANDES */

  /* HALT */
  PT_HALT,                     /* 66 */
  PT_HALT_PAN,                 /* 67 */
  PT_HALT_TILT,                /* 68 */

  /* SPEED MODE */
  PT_GET_CNTRL_MODE,           /* 69 */
  PT_VELOCITY_CNTRL,           /* 70 */
  PT_POSITION_CNTRL,           /* 71 */

  /* RESET MODE */
  PT_RESET_PAN,                /* 72 */
  PT_RESET_TILT,               /* 73 */

  PT_RESET_PTU_CONTROLER,      /* 74 */

  PT_NB_CMD                    /* 75 */

} PT_CMD;


/*----------------------------------------------------------------------*/
/* CODES CHAINES */
/* Verifier la coherence avec ptStringsDesc dans ptDriverPrivate.h */

typedef enum PT_STRINGS_DECODE {
  PT_REGULAR_POWER,
  PT_LOW_POWER,
  PT_HIGH_POWER,
  PT_POWER_OFF,
  PT_ENABLE,
  PT_DISABLE,
  PT_NB_STRINGS_DECODE
} PT_STRINGS_DECODE;

/*----------------------------------------------------------------------
 *    DESCRIPTION DES MESSAGES D'ERREUR  (XXX A COMPLETER XXXXX)
 *
 *    (Si modif, maintenir coherence avec ptErrDesc dans ptDriverPrivate.h)
 */

typedef enum PT_ERR {
  PT_OUT_OF_RANGE,
  PT_SMALL_PAN_VALUE,
  PT_BIG_PAN_VALUE,
  PT_SMALL_TILT_VALUE,
  PT_BIG_TILT_VALUE,
  PT_SMALL_PAN_SPEED,
  PT_SMALL_TILT_SPEED,
  PT_ILLEGAL_COMMAND,
  PT_ILLEGAL_ACCEL,
  PT_SYS_ERR,
  PT_STOP_ENDS,
  PT_TIMEOUT_ERR,
  PT_UNKNOWN_ERR,
  PT_NO_ERR,
  PT_NB_ERR   /* Le nombre d'erreurs */
}PT_ERR;

/*----------------------------------------------------------------------
 * arguments struct
 */
typedef struct PT_DRIVER_ARGS {
  const char *device;
  int baudRate;
  PT_DRIVER_STR *sdiPtr;
} PT_DRIVER_ARGS;

#endif /* PT_DRIVER_H */
